#!/usr/bin/env python


func_and_ctes = '''
# =============================
#  New functions and constants
# =============================
def isiterable(obj):
    """
    Checks if the given object *obj* is iterable (excluding strings).
    Tested with: list, tuple, dict, set, numpy arrays
    """
    return hasattr(obj, "__iter__")

# list of acceptable floats
try:
    import numpy
    _floats = (float, numpy.float64, numpy.float32, numpy.float)
    _complexs = (complex, numpy.complex)
    hasnumpy = True
except ImportError, err:
    # numpy not available, no problem
    _floats = float
    _complexs = complex
    hasnumpy = False


################################

'''


safes = """def safe_len(msg,v):
    if not isiterable(v):
        raise ValueError(msg)
    return len(v)

#
# safe matrix size
# * returns number of columns
# * throws ValueError if 'v' is not rectangular matrix
#   (list of lists of same size)
#   it uses 'msg' parameter to generate error message
#
def safe_cols(msg,v):
    if not isiterable(v):
        raise ValueError(msg)
    if len(v)==0:
        return 0
    if not isiterable(v[0]):
        raise ValueError(msg)
    cols = len(v[0])
    for x in v:
        if not isiterable(x):
            raise ValueError(msg)
        if len(x)!=cols:
            raise ValueError(msg)
    return cols

#
# safe matrix size
# * returns number of rows
# * throws ValueError if 'v' is not rectangular matrix
#   (list of lists of same size)
#   it uses 'msg' parameter to generate error message
#
def safe_rows(msg,v):
    if not isiterable(v):
        raise ValueError(msg)
    if len(v)==0:
        return 0
    if not isiterable(v[0]):
        raise ValueError(msg)
    cols = len(v[0])
    for x in v:
        if not isiterable(x):
            raise ValueError(msg)
        if len(x)!=cols:
            raise ValueError(msg)
    return len(v)

"""


conversions = """
def castto(x, dtype, errmsg=None):
    '''
    Cast x to dtype. x could be a single value, a vector or a matrix.
    This function do NOT check the dimension of x, but guess it and try to
    cast it to the given type anyway. This check must be done prior to calling
    this function.
    '''
    if isiterable(x):
        # vector or matrix
        if hasnumpy:
            # if numpy is avalaible, speed-up the process
            try:
                res = numpy.array(x, dtype=dtype)
            except ValueError, err:
                if errmsg is None:
                    errmsg = err
                raise ValueError(errmsg)
        elif isiterable(x[0]):
            # matrix
            res = []
            for vect in x:
                try:
                    res.append(map(dtype, vect))
                except ValueError, err:
                    if errmsg is None:
                        errmsg = err
                    raise ValueError(errmsg)
        else:
            # vector
            try:
                res = map(dtype, x)
            except ValueError, err:
                if errmsg is None:
                    errmsg = err
                raise ValueError(errmsg)
    else:
        try:
            res = dtype(x)
        except ValueError, err:
            if errmsg is None:
                errmsg = err
            raise ValueError(errmsg)
    return res

#
# conversion from list to x-vector:
#
# x     x-vector.
# v     list
# dt    datatype
# mode  one of the modes:
#       * X_CREATE -  x is assumed to be uninitialized
#       * X_SET -     x is assumed to be initialized; old contents is freed
#       * X_REWRITE - x is assumed to have same size as v, exception is thrown otherwise
#                     it is rewritten without reallocation of memory;
#
def x_from_list(x, v, dt, mode, errmsg=None):
    #
    # check types
    #
    if dt==DT_BOOL:
        if errmsg is None:
            errmsg = "can't cast to bool_vector"
        dtype = bool
        elemsize = SIZE_BOOL
    elif dt==DT_INT:
        if errmsg is None:
            errmsg = "can't cast to int_vector"
        dtype = int
        elemsize = SIZE_INT
    elif dt==DT_REAL:
        if errmsg is None:
            errmsg = "can't cast to real_vector"
        dtype = float
        elemsize = SIZE_REAL
    elif dt==DT_COMPLEX:
        if errmsg is None:
            errmsg = "can't cast to complex_vector"
        dtype = complex
        elemsize = SIZE_COMPLEX

    # check dimensions
    if hasnumpy:
        # better than using 'isiterable', because we test the shape of v
        ndim = len(numpy.shape(v))
        if ndim == 0:
            # cast a single value into a vector
            v = [v]
        elif ndim > 1:
            # do not accept matrix (one could reshape it...)
            raise ValueError(errmsg)
    else:
        if not isiterable(x):
            raise ValueError(errmsg)

    # Cast to desired type
    v = castto(v, dtype, errmsg)

    #
    # allocation
    #
    if mode==X_CREATE:
        x.datatype = dt
        x.cnt = len(v)
        x.owner = OWN_AE
        x.ptr.p_ptr = x_malloc(elemsize*x.cnt)
        x.last_action = 1
    elif mode==X_SET:
        if x.owner==OWN_AE:
            x_free(x.ptr.p_ptr)
        x.datatype = dt
        x.cnt = len(v)
        x.owner = OWN_AE
        x.ptr.p_ptr = x_malloc(elemsize*x.cnt)
        x.last_action = 3
    elif mode==X_REWRITE:
        if x.datatype!=dt:
            raise RuntimeError("Trying to rewrite vector - types don't match")
        if len(v)!=x.cnt:
            raise RuntimeError("Trying to rewrite vector - sizes don't match")
        x.last_action = 2

    #
    # copy
    #
    if dt==DT_BOOL:
        cnt = x.cnt
        i = 0
        while i<cnt:
            x.ptr.p_bool[i] =   bool(v[i])
            i += 1
    elif dt==DT_INT:
        cnt = x.cnt
        i = 0
        while i<cnt:
            x.ptr.p_int[i] =    int(v[i])
            i += 1
    elif dt==DT_REAL:
        cnt = x.cnt
        i = 0
        while i<cnt:
            x.ptr.p_real[i] = float(v[i])
            i += 1
    elif dt==DT_COMPLEX:
        cnt = x.cnt
        i = 0
        while i<cnt:
            tmp = complex(v[i])
            x.ptr.p_complex[i].x = tmp.real
            x.ptr.p_complex[i].y = tmp.imag
            i += 1

    return


#
# conversion from list of lists to x-matrix:
#
# x     x-matrix.
# v     list
# dt    datatype
# mode  one of the modes:
#       * X_CREATE -  x is assumed to be uninitialized
#       * X_SET -     x is assumed to be initialized; old contents is freed
#       * X_REWRITE - x is assumed to have same size as v, exception is thrown otherwise
#                     it is rewritten without reallocation of memory;
#
def x_from_listlist(x, v, dt, mode, errmsg=None):
    #
    # check types
    #
    if dt==DT_BOOL:
        if errmsg is None:
            errmsg = "can't cast to bool_matrix"
        dtype = bool
        elemsize = SIZE_BOOL
    elif dt==DT_INT:
        if errmsg is None:
            errmsg = "can't cast to int_matrix"
        dtype = int
        elemsize = SIZE_INT
    elif dt==DT_REAL:
        if errmsg is None:
            errmsg = "can't cast to real_matrix"
        dtype = float
        elemsize = SIZE_REAL
    elif dt==DT_COMPLEX:
        if errmsg is None:
            errmsg = "can't cast to complex_matrix"
        dtype = complex
        elemsize = SIZE_COMPLEX

    # check dimensions
    if hasnumpy:
        # better than using 'isiterable', because we test the shape of v
        if len(numpy.shape(v)) != 2:
            raise ValueError(errmsg)
    else:
        if not isiterable(v):
            raise ValueError(errmsg)
            if not isiterable(v[0]):
                raise ValueError(errmsg)
            ncols = len(v[0])
            for row in v:
                if len(row) != ncols:
                    raise ValueError(errmsg)

    # Cast to desired type
    v = castto(v, dtype, errmsg)

    #
    # determine size
    #
    rows = len(v)
    if rows>0:
        cols = len(v[0])
    else:
        cols = 0
    if cols==0:
        rows = 0

    #
    # allocation
    #
    if mode==X_CREATE:
        x.datatype = dt
        x.cols = cols
        x.rows = rows
        x.stride = cols
        x.owner = OWN_AE
        x.ptr.p_ptr = x_malloc(elemsize*x.stride*x.rows)
        x.last_action = 1
    elif mode==X_SET:
        if x.owner==OWN_AE:
            x_free(x.ptr.p_ptr)
        x.cols = cols
        x.rows = rows
        x.stride =cols
        x.owner = OWN_AE
        x.ptr.p_ptr = x_malloc(elemsize*x.stride*x.rows)
        x.last_action = 3
    elif mode==X_REWRITE:
        if x.datatype!=dt:
            raise RuntimeError("Trying to rewrite matrix - types don't match")
        if rows!=x.rows or cols!=x.cols:
            raise RuntimeError("Trying to rewrite vector - sizes don't match")
        x.last_action = 2

    #
    # copy
    #
    offs = 0
    endoffs = x.stride-x.cols
    for p in v:
        for q in p:
            if dt==DT_BOOL:
                x.ptr.p_bool[offs] = bool(q)
            elif dt==DT_INT:
                x.ptr.p_int[offs]  = int(q)
            elif dt==DT_REAL:
                x.ptr.p_real[offs] = float(q)
            elif dt==DT_COMPLEX:
                tmp = complex(q)
                x.ptr.p_complex[offs].x = tmp.real
                x.ptr.p_complex[offs].y = tmp.imag
            offs = offs+1
        offs = offs+endoffs
    return


#
# conversion from x-vector to Python vector
#
# Function takes only one parameter - x, x-vector,
# which is NOT freed after use.
#
def list_from_x(x):
    if x.cnt==0:
        return []
    r = [0]*x.cnt
    i = 0
    if x.datatype==DT_BOOL:
        while i<x.cnt:
            r[i] = bool(x.ptr.p_bool[i])
            i += 1
    elif x.datatype==DT_INT:
        while i<x.cnt:
            r[i] = x.ptr.p_int[i]
            i += 1
    elif x.datatype==DT_REAL:
        while i<x.cnt:
            r[i] = x.ptr.p_real[i]
            i += 1
    elif x.datatype==DT_COMPLEX:
        while i<x.cnt:
            r[i] = complex(x.ptr.p_complex[i].x, x.ptr.p_complex[i].y)
            i += 1
    return r


#
# conversion from x-matrix to Python matrix
#
# Function takes only one parameter - x, x-matrix,
# which is NOT freed after use.
#
def listlist_from_x(x):
    if x.cols==0 or x.rows==0:
        return [[]]
    r = create_real_matrix(x.rows, x.cols)
    offs = 0
    endoffs = x.stride-x.cols
    m = x.rows
    n = x.cols
    dt = x.datatype
    i = 0
    while i<m:
        j = 0
        while j<n:
            if dt==DT_BOOL:
                r[i][j] = bool(x.ptr.p_bool[offs])
            elif dt==DT_INT:
                r[i][j] = x.ptr.p_int[offs]
            elif dt==DT_REAL:
                r[i][j] = x.ptr.p_real[offs]
            elif dt==DT_COMPLEX:
                r[i][j] = complex(x.ptr.p_complex[offs].x, x.ptr.p_complex[offs].y)
            offs += 1
            j += 1
        offs += endoffs
        i += 1
    return r



#
# this function copies x-vector to previously allocated list
# which should be large enough to store x-vector.
#
# invalid access to list is generated if list is too small.
# x-vector is not freed after use.
#
def copy_x_to_list(x,r):
    if x.cnt==0:
        return
    i = 0
    if x.datatype==DT_BOOL:
        while i<x.cnt:
            r[i] = bool(x.ptr.p_bool[i])
            i += 1
    elif x.datatype==DT_INT:
        while i<x.cnt:
            r[i] = x.ptr.p_int[i]
            i += 1
    elif x.datatype==DT_REAL:
        while i<x.cnt:
            r[i] = x.ptr.p_real[i]
            i += 1
    elif x.datatype==DT_COMPLEX:
        while i<x.cnt:
            r[i] = complex(x.ptr.p_complex[i].x, x.ptr.p_complex[i].y)
            i += 1
    return
"""